var sdk = (function() {
  var callbackId = 0;

  var _extend = function(obj1, obj2) {
    for (var i in obj2) {
      obj1[i] = obj2[i];
    }
    return obj1;
  };

  var _containsValue = function(obj, val) {
    for (var prop in obj) {
      if (obj.hasOwnProperty(prop)) {
        if (obj[prop] === val) {
          return true;
        }
      }
    }
    return false;
  };

  var _callSdk = function(endpoint, options, callback) {
    var optionsString = JSON.stringify(options);
    var callbackName;

    if (callback && typeof callback === 'function') {
      var generatedCallbackName = 'callback' + callbackId++;

      window[generatedCallbackName] = function(data, error) {
          callback(data, error);
          delete window[generatedCallbackName];
      };
      callbackName = generatedCallbackName;
    }

    android.callSdk(endpoint, optionsString, callbackName);
  };

  var _report = function(msg, caught, src, line, col, stack) {
    try {
      var data = {
          message: msg,
          caught: caught,
          source: src,
          lineNumber: line,
          columnNumber: col,
          stackTrace: stack,
          timestamp: new Date().getTime()
      };
      _callSdk("reportError", data, null);
    } catch(e) {
      // Ignore errors thrown by the error reporter to avoid recursive error
      // throwing
      // NOTE(amullings): In all browsers tested, it seems to be the case that
      // errors thrown from window.onerror will not trigger another error event,
      // so this is mostly for safety.
    }
  };

  window.onerror = function(msg, src, line, col, err) {
    var stack = (err && err instanceof Error) ? err.stack : undefined;
    _report(msg, false, src, line, col, stack);
  };

  var _stackRegex = /^.+@(.+):(\d+):(\d+)$/;
  var reportError = function(err) {
      var errObj = err;
      if (err && !(err instanceof Error)) {
        errObj = new Error(err.error ? err.error : err);
      }
      if (errObj && errObj instanceof Error) {
        var stack = errObj, src = undefined, line = 0, col = 0;
        if (typeof stack == 'string') {
          var match = _stackRegex.exec(stack.split('\\n')[0]);
          if (match) {
            src = match[1];
            line = match[2];
            col = match[3];
          }
        }
        _report(errObj.message, true, src, line, col, stack);
      }
    };

  var close = function() {
    _callSdk("close", {}, null);
  };

  var track = function(opts, cb) {
    _callSdk("track", opts, cb);
  };

  var socialSites = {
    FACEBOOK: 'facebook',
    TWITTER: 'twitter'
  };

  var nativePost = function(opts, cb) {
    if (typeof opts['site'] !== 'string' || !_containsValue(socialSites, opts['site'])) {
      throw new Error('nativePost: invalid site');
    }
    if (opts.hasOwnProperty('message') && typeof opts['message'] !== 'string') {
      throw new Error('nativePost: invalid message');
    }
    if (opts.hasOwnProperty('link') && typeof opts['link'] !== 'string') {
      throw new Error('nativePost: invalid url');
    }
    if (opts.hasOwnProperty('photo') && typeof opts['photo'] !== 'string') {
      throw new Error('nativePost: invalid photo');
    }
    _callSdk("nativePost", opts, cb);
  };

  var getNativeNetworks = function(cb) {
    _callSdk("getNativeNetworks", {}, cb);
  };
  var takePhoto = function(opts, cb) {
    _callSdk("takePhoto", opts, cb);
  };
  var postPhoto = function(opts, cb) {
      _callSdk("postPhoto", opts, cb);
  };
  var phoneCall = function(opts, cb) {
    _callSdk("phoneCall", opts, cb);
  };
  var addToContacts = function(opts, cb) {
    _callSdk("addToContacts", opts, cb);
  };
  var addToContactsInBackground = function(opts, cb) {
    _callSdk("addToContactsInBackground", opts, cb);
  };
  var share = function(opts, cb) {
    _callSdk("share", opts, cb);
  }
  var sendTextMessage = function(opts, cb) {
    _callSdk("sendTextMessage", opts, cb);
  }
  var alert = function(opts, cb) {
    _callSdk("alert", opts, cb);
  }
  var screenshot = function(opts, cb) {
    _callSdk("screenshot", opts, cb)
  }
  var passbook = function(opts, cb) {
    _callSdk("passbook", opts, cb);
  }

  var module = {};
  module.reportError = reportError;
  module.close = close;
  module.track = track;
  module.socialSites = socialSites;
  module.nativePost = nativePost;
  module.getNativeNetworks = getNativeNetworks;
  module.addToContacts = addToContacts;
  module.share = share;
  module.alert = alert;
  module.screenshot = screenshot;
  // INJECT
  return module;
})();